@@ -6,14 +6,15 @@ These ones pass, just as you'd hope!
 """
 from __future__ import absolute_import, division, print_function
 
-from hypothesis import given, settings
-import hypothesis.strategies as st
 import hypothesis.extra.numpy as npst
+import hypothesis.strategies as st
+from hypothesis import given, settings
 
 import xarray as xr
 
 # Run for a while - arrays are a bigger search space than usual
-settings.deadline = None
+settings.register_profile("ci", deadline=None)
+settings.load_profile("ci")
 
 
 an_array = npst.arrays(
