@@ -17,10 +17,9 @@ class InMemoryDataStore(AbstractWritableDataStore):
     This store exists purely for internal testing purposes.
     """
 
-    def __init__(self, variables=None, attributes=None, writer=None):
+    def __init__(self, variables=None, attributes=None):
         self._variables = OrderedDict() if variables is None else variables
         self._attributes = OrderedDict() if attributes is None else attributes
-        super(InMemoryDataStore, self).__init__(writer)
 
     def get_attrs(self):
         return self._attributes
